@@ -2,6 +2,14 @@ use strict;
 use warnings;
 use Future::AsyncAwait;
 
+# Try to load Future::IO for loop-agnostic sleep, fall back to immediate if not available
+my $HAS_FUTURE_IO = eval { require Future::IO; 1 };
+
+sub maybe_sleep {
+    my ($seconds) = @_;
+    return $HAS_FUTURE_IO ? Future::IO->sleep($seconds) : Future->done;
+}
+
 async sub watch_sse_disconnect {
     my ($receive) = @_;
 
@@ -29,10 +37,9 @@ async sub app {
         { event => 'done', data => 'finished' },
     );
 
-    my $loop = IO::Async::Loop->new;
     for my $msg (@events) {
         last if $disconnect->is_ready;
-        await $loop->delay_future(after => 1);
+        await maybe_sleep(1);
         await $send->({ type => 'sse.send', %$msg });
     }
 
