@@ -643,78 +643,6 @@ when C<strict> mode would be enabled) will cause the Future to fail.
         warn "Failed to send response: $e";
     }
 
-=head1 RECIPES
-
-=head2 Background Tasks
-
-Run tasks after the response is sent. There are three patterns depending
-on what kind of work you're doing:
-
-=head3 Pattern 1: Fire-and-Forget Async I/O (Non-Blocking)
-
-For async operations (HTTP calls, database queries using async drivers),
-call them without C<await>, add C<< ->on_fail() >> for error handling,
-then C<< ->retain() >> to prevent the "lost future" warning:
-
-    await $res->json({ status => 'queued' });
-
-    # Fire-and-forget with error handling
-    # IMPORTANT: Always add on_fail() before retain() to avoid silent failures
-    send_async_email($user)
-        ->on_fail(sub { warn "Email failed: @_" })
-        ->retain();
-    log_to_analytics($event)
-        ->on_fail(sub { warn "Analytics failed: @_" })
-        ->retain();
-
-B<Warning:> Using C<< ->retain() >> alone silently swallows errors.
-
-B<Note:> If you're writing middleware or server extensions that inherit from
-L<IO::Async::Notifier>, prefer C<< $self->adopt_future($f) >> instead of
-C<< ->retain() >>. The C<adopt_future> method properly tracks futures and
-propagates errors to the notifier's error handling.
-
-=head3 Pattern 2: Blocking/CPU Work (IO::Async::Function)
-
-For blocking operations (sync libraries, CPU-intensive work), use
-L<IO::Async::Function> to run in a subprocess:
-
-    use IO::Async::Function;
-
-    my $worker = IO::Async::Function->new(
-        code => sub {
-            my ($data) = @_;
-            # This runs in a CHILD PROCESS - can block safely
-            sleep 5;  # Won't block event loop
-            return process($data);
-        },
-    );
-    $res->loop->add($worker);
-
-    await $res->json({ status => 'processing' });
-
-    # Fire-and-forget in subprocess
-    my $f = $worker->call(args => [$data]);
-    $f->on_done(sub { warn "Done: @_\n" });
-    $f->on_fail(sub { warn "Error: @_\n" });
-    $f->retain();
-
-=head3 Pattern 3: Quick Sync Work (loop->later)
-
-For very fast sync operations (logging, incrementing counters):
-
-    await $res->json({ status => 'ok' });
-
-    $res->loop->later(sub {
-        log_request();  # Must be FAST (<10ms)
-    });
-
-B<WARNING:> Any blocking code in C<loop-E<gt>later> blocks the entire
-event loop. No other requests can be processed. Use IO::Async::Function
-for anything that might take time.
-
-See also: C<examples/background-tasks/app.pl>
-
 =head1 SEE ALSO
 
 L<PAGI>, L<PAGI::Request>, L<PAGI::Server>
@@ -741,12 +669,6 @@ sub new {
     return $self;
 }
 
-sub loop {
-    my ($self) = @_;
-    require IO::Async::Loop;
-    return IO::Async::Loop->new;
-}
-
 sub status {
     my ($self, $code) = @_;
     croak("Status must be a number between 100-599")
